<?php
/**
 * Plugin cid : Fonctions diverses
 * Copyright (c) Christophe IMBERTI
 * Licence Creative commons by-nc-sa
 */

if (!defined('_ECRIRE_INC_VERSION')) return;

include_spip('inc/editer');
include_spip('inc/autoriser');
include_spip('inc/puce_statut');



/**
 * Filtre les statuts utilisable selon les droits de publication
 * 
 * @param array $desc
 * @param bool $publiable
 * @return array
 */
function cid_lister_statuts_proposes($id_objet,$statut,$desc,$publiable = true){
	if (!isset($desc['statut_textes_instituer']))
		return false;

	$l = $desc['statut_textes_instituer'];

// Sous SPIP 3.0 une nouvelle rubrique a le statut "new"
// Sous SPIP 3.1 et SPIP 3.2 une nouvelle rubrique a le statut "prepa"
	$unset_prive = false;
	$unset_new = false;
	$unset_0 = false;
	$unset_prepa = false;
        
	if ($statut=='new'){
		$unset_prive = true;
		$unset_0 = true;
        	$unset_prepa = true;
	} elseif ($statut=='prive') {
		$unset_new = true;
		$unset_0 = true;
        	$unset_prepa = true;
	} elseif ($statut=='0') {
		$unset_prive = true;
		$unset_new = true;
        	$unset_prepa = true;
	} elseif ($statut=='prepa') {
		$unset_prive = true;
		$unset_new = true;
		$unset_0 = true;
	}

	// Ne pas proposer de depublier une rubrique qui a des objets publies
	if ($statut=='publie') {
		if (cid_rubrique_avec_objet_publie(intval($id_objet))){
			$unset_prive = true;
			$unset_new = true;
			$unset_0 = true;
                	$unset_prepa = true;
		} else {
			$unset_new = true;
			$unset_0 = true;
                        if (spip_version()>=3.1){
        			$unset_prive = true;
                        } else {
                        	$unset_prepa = true;
                        }
                            
		}
	}

	if ($unset_prive AND isset($l['prive']))
		unset($l['prive']);
	if ($unset_new AND isset($l['new']))
		unset($l['new']);
	if ($unset_0 AND isset($l['0']))
		unset($l['0']);
	if ($unset_prepa AND isset($l['prepa']))
		unset($l['prepa']);
	
	if (!$publiable AND isset($l['publie']))
		unset($l['publie']);

	return $l;
}

/**
 * Chargement du formulaire instituer rubrique
 *
 * @uses formulaires_editer_objet_charger()
 * 
 * @param string $objet
 *     Type d'objet
 * @param int $id_objet
 *     Identifiant de l'objet
 * @param string $retour
 *     URL de redirection après le traitement
 * @param bool $editable
 *     Indique si le statut est éditable ou non.
 *     Dans tous les cas, si l'on n'a pas la permission de modifier l'objet,
 *     cette option sera mise à false.
 * @return array|bool
 *     Environnement du formulaire ou false si aucun affichage à faire.
 */
function formulaires_cid_instituer_rubrique_charger_dist($objet,$id_objet,$retour="",$editable = true){
	$editable = ($editable?true:false);

	$desc = cid_desc_rubrique();

	if (!isset($desc['statut_textes_instituer']))
		return false;

	if (!autoriser('modifier', $objet, $id_objet))
		$editable = false;

	// charger le contenu de l'objet
	// dont son champ statut
	$v = formulaires_editer_objet_charger($objet,$id_objet,0,0,'','');

	$publiable = true;
	$statuts = cid_lister_statuts_proposes($id_objet,$v['statut'],$desc);
	// tester si on a le droit de publier, si un statut publie existe
	if (isset($statuts['publie'])){
		if (!autoriser('instituer', $objet, $id_objet, null, array('statut'=>'publie'))){
			if ($v['statut'] == 'publie')
				$editable = false;
			else
				$publiable = false;
		}
	}
	$statuts = cid_lister_statuts_proposes($id_objet,$v['statut'],$desc, $editable?$publiable:true);
	if (count($statuts)==1  AND isset($statuts[$v['statut']])){
		$editable = false;
	}

	$valeurs = array(
		'editable' => $editable,
		'statut' => $v['statut'],
		'_objet' => $objet,
		'_id_objet' => $id_objet,
		'_statuts' => $statuts,
		'_publiable' => $publiable,
		'_label' => isset($desc['texte_changer_statut'])?$desc['texte_changer_statut']:'texte_article_statut',
		'_aide' => isset($desc['aide_changer_statut'])?$desc['aide_changer_statut']:'',
		'_hidden' => "<input type='hidden' name='statut_old' value='".$v['statut']."' />",
	);

	#if (!count($valeurs['statuts']))
	return $valeurs;
}

/**
 * Vérifications du formulaire instituer rubrique
 *
 * @uses formulaires_editer_objet_charger()
 * 
 * @param string $objet
 *     Type d'objet
 * @param int $id_objet
 *     Identifiant de l'objet
 * @param string $retour
 *     URL de redirection après le traitement
 * @param bool $editable
 *     Indique si le statut est éditable ou non.
 * @return array
 *     Tableau des erreurs
 */
function formulaires_cid_instituer_rubrique_verifier_dist($objet,$id_objet,$retour="",$editable = true){
	$erreurs = array();
	// charger le contenu de l'objet
	// dont son champ statut
	$v = formulaires_editer_objet_charger($objet,$id_objet,0,0,'','');

	if ($v['statut']!==_request('statut_old'))
		$erreurs['statut'] = _T('instituer_erreur_statut_a_change');
	else {
		$desc = cid_desc_rubrique();

		$publiable = true;
		if (isset($v['id_rubrique'])
			AND !autoriser('publierdans', 'rubrique', $v['id_rubrique'])) {
			$publiable = false;
		}
		$l = cid_lister_statuts_proposes($id_objet,$v['statut'],$desc,$publiable);
		$statut = _request('statut');
		if (!isset($l[$statut])
		  OR !autoriser('instituer',$objet,$id_objet,'',array('statut'=>$statut)))
			$erreurs['statut'] = _T('instituer_erreur_statut_non_autorise');
	}

	return $erreurs;
}

/**
 * Traitements du formulaire instituer rubrique
 * 
 * @param string $objet
 *     Type d'objet
 * @param int $id_objet
 *     Identifiant de l'objet
 * @param string $retour
 *     URL de redirection après le traitement
 * @param bool $editable
 *     Indique si le statut est éditable ou non.
 * @return array
 *     Retour des traitements
 */
function formulaires_cid_instituer_rubrique_traiter_dist($objet,$id_objet,$retour="",$editable = true){

	$id_rubrique = $id_objet;
	$statut = _request('statut');
	
	if ($id_rubrique = intval($id_rubrique)) {
		if (in_array($statut, array('publie','prive','new','0','prepa'))) {
			// cf autorisations dans action/editer_rubrique
			if (autoriser('modifier', 'rubrique', $id_rubrique)) {

				// Ne pas depublier une rubrique qui a des objets publies
				if ($statut=='publie' OR !cid_rubrique_avec_objet_publie(intval($id_rubrique))){
					$action = '';
					$row = sql_fetsel('titre,statut', 'spip_rubriques', 'id_rubrique='.$id_rubrique);
					$statut_ancien = $row['statut'];
					$titre_rubrique = $row['titre'];
					$champs = array('statut'=>$statut, 'date'=>'0000-00-00 00:00:00');
					
					sql_updateq('spip_rubriques', $champs, "id_rubrique=$id_rubrique");

					// invalider les caches marques de cette rubrique
					include_spip('inc/invalideur');
					suivre_invalideur("id='rubrique/$id_rubrique'");
					// et celui de menu_rubriques 
					effacer_meta("date_calcul_rubriques");
					
					// tracer
					if ($statut=='publie' AND $statut_ancien!='publie')
						$action = "publication directe rubrique";
					elseif ($statut!='publie' AND $statut_ancien=='publie')
						$action = "depublication directe rubrique";

					if ($action){
						if (defined('_DIR_PLUGIN_CITRACE')){
							if ($citrace = charger_fonction('citrace', 'inc')) {
								include_spip('inc/texte');
								$titre_rubrique = interdire_scripts(supprimer_numero($titre_rubrique));
								$message = '('.$titre_rubrique.')';							
								$citrace('rubrique', $id_rubrique, $action, $message, $id_rubrique);
							}
						}
					}
				}			
			}
		}
	}
	
	
	$res = array('message_ok'=>_T('info_modification_enregistree'));

	if ($retour)
		$res['redirect'] = $retour;
	set_request('statut');

	return $res;
}


function cid_desc_rubrique(){
	$desc = array();
	
	$desc['statut_titres']['new'] = 'texte_statut_en_cours_redaction';
	$desc['statut_titres']['prive'] = 'texte_statut_en_cours_redaction';
	$desc['statut_titres']['0'] = 'texte_statut_en_cours_redaction';
	$desc['statut_titres']['publie'] = 'texte_statut_publie';
	$desc['statut_titres']['prepa'] = 'texte_statut_en_cours_redaction';
	$desc['statut_textes_instituer']['new'] = 'texte_statut_en_cours_redaction';
	$desc['statut_textes_instituer']['prive'] = 'texte_statut_en_cours_redaction';
	$desc['statut_textes_instituer']['0'] = 'texte_statut_en_cours_redaction';
	$desc['statut_textes_instituer']['prepa'] = 'texte_statut_en_cours_redaction';
	$desc['statut_textes_instituer']['publie'] = 'texte_statut_publie';

	$desc['statut_images'] = array(
					'puce-preparer-8.png',
					'new'=>'puce-preparer-8.png',
					'prive'=>'puce-preparer-8.png',
					'prepa'=>'puce-preparer-8.png',
					'0'=>'puce-preparer-8.png',
					'publie'=>'puce-publier-8.png'
				);

	$desc['statut'] = array(
					array(
						'champ' => 'statut',
						'publie' => 'publie',
						'previsu' => 'publie,prive,new,0,prepa',
						'exception' => array('statut','tout')
					),
				);

	$desc['texte_changer_statut'] = '';

				
	return $desc;
}


function cid_puce_statut_rubrique($statut, $atts='') {
	$desc = cid_desc_rubrique();
	$src = $desc['statut_images'][$statut];
	
	if (!$src) return '';
	return http_img_pack($src, statut_titre('rubrique',$statut), $atts);
}

function cid_rubrique_avec_objet_publie($id_rubrique) {

//	if (is_null($date)) {
		$date = date('Y-m-d H:i:s');
//	}
	$postdates = ($GLOBALS['meta']["post_dates"] == "non") ?
		" AND date <= ".sql_quote($date) : '';

	if (!$id_rubrique=intval($id_rubrique))
		return false;
	
	// verifier qu'elle existe et est bien publiee
	$r = sql_fetsel('id_rubrique,statut','spip_rubriques',"id_rubrique=".intval($id_rubrique));
	if (!$r OR $r['statut']!=='publie')
		return false;

	// On met le nombre de chaque type d'enfants dans un tableau
	// Le type de l'objet est au pluriel
	$compte = array(
		'articles' => sql_countsel("spip_articles",  "id_rubrique=".intval($id_rubrique)." AND statut='publie'$postdates"),
		'rubriques' => sql_countsel("spip_rubriques",  "id_parent=".intval($id_rubrique)." AND statut='publie'"),
		'documents' => sql_countsel("spip_documents_liens",  "id_objet=".intval($id_rubrique)." AND objet='rubrique'")
	);
	
	// S'il y a au moins un objet publie
	foreach($compte as $objet => $n)
		if ($n)
			return true;

	return false;			
}

?>