<?php
/**
 * Plugin Configurateur de squelettes
 * Copyright (c) Christophe IMBERTI
 * Licence Creative commons by-nc-sa
 */

if (!defined("_ECRIRE_INC_VERSION")) return;


// Assurer la double compatibilite HTML 4.01 et XHTML 1.0
// - enlever le cas echeant les />
// - ou bien transformer le cas echeant les <br> saisis manuellement dans le texte d'un article
// Accessibilite : indiquer dans le title de la page si la reponse du formulaire est OK ou erreur 
function ciparam_affichage_final($texte) {
        $msg_erreur_done = false;

        if (empty($texte)){
            return '';
        }        
// On est desormais en HTML 5
// Par ailleurs, ces lignes generent une incompatibilite avec le plugin crayons                   
/*        
        // Assurer la double compatibilite HTML 4.01 et XHTML 1.0
	if (defined('_CIPARAM_DOCTYPE') AND _CIPARAM_DOCTYPE=='xhtml') {
		if (strpos($texte, '<br>')!==false)
			$texte = str_replace('<br>','<br />',$texte);
		if (strpos($texte, '<BR>')!==false)
			$texte = str_replace('<BR>','<br />',$texte);
	} else {
		if (strpos($texte, '/>')!==false AND strpos($texte, '<?xml')===false)
			$texte = str_replace(array(' />','/>'),'>',$texte);
	}
*/        
        // Accessibilite : indiquer dans le title de la page si la reponse du formulaire est OK ou erreur 
        if (strpos($texte, 'formulaire_erreur')!==false){
            $texte = str_replace('<title>','<title>'._T('ciparam:formulaire_erreur').' - ',$texte);
            $msg_erreur_done = true;
        } elseif (strpos($texte, 'formulaire_ok')!==false){
            $texte = str_replace('<title>','<title>'._T('ciparam:formulaire_ok').' - ',$texte);
        }
        
        // Cas particulier du formulaire forum de SPIP
        // qui utilise AJAX, un mecanisme de previsualisation, mais pas 'message_erreur'
        // Mis a jour suite a la surcharge du formulaire forum
        if (strpos($texte, "formulaire_forum")!==false){
//            $texte = str_replace("formulaire_forum ajax","formulaire_forum",$texte);
                        
            if (strpos($texte, '<div class="previsu">')!==false){
                $texte = str_replace('<title>','<title>'._T('previsualisation').' - ',$texte);
            } elseif (!$msg_erreur_done AND strpos($texte, 'erreur_message')!==false){
                $texte = str_replace('<title>','<title>'._T('ciparam:formulaire_erreur').' - ',$texte);
            }            
        }

	return $texte;
}

/**
 * Optimiser la base de donnee en supprimant les liens entre raccourcis et objets supprimes
 *
 * @param int $n
 * @return int
 */
function ciparam_optimiser_base_disparus($n){

	// les liens dans spip_ci_raccourcis_articles
	$res = sql_select("lien.id_article AS id",
	 	        "spip_ci_raccourcis_articles AS lien
		        LEFT JOIN spip_articles AS articles
		          ON lien.id_article=articles.id_article",
			"articles.id_article IS NULL");

	$t = optimiser_sansref('spip_ci_raccourcis_articles', 'id_article', $res);

	// les liens dans spip_ci_raccourcis_rubriques
	$res = sql_select("lien.id_rubrique AS id",
	 	        "spip_ci_raccourcis_rubriques AS lien
		        LEFT JOIN spip_rubriques AS rubriques
		          ON lien.id_rubrique=rubriques.id_rubrique",
			"rubriques.id_rubrique IS NULL");

	$t = optimiser_sansref('spip_ci_raccourcis_rubriques', 'id_rubrique', $res);
        
	// les liens dans spip_ci_raccourcis_syndic
	$res = sql_select("lien.id_syndic AS id",
	 	        "spip_ci_raccourcis_syndic AS lien
		        LEFT JOIN spip_syndic AS syndic
		          ON lien.id_syndic=syndic.id_syndic",
			"syndic.id_syndic IS NULL");

	$t = optimiser_sansref('spip_ci_raccourcis_syndic', 'id_syndic', $res);
        
    return $n;
}


function ciparam_affichage_final_prive($texte) {
    if (defined('_CIPARAM_CHRONO') AND _CIPARAM_CHRONO=='oui'){
        if (test_espace_prive()){
            $code = "<div>Temps de traitement : ".round(spip_timer('cichrono',true)/1000,3)." seconde(s)</div>";

            // inserer avant le </body> fermant si on peut, a la fin de la page sinon
            if (($p = strpos($texte, '</body>')) !== false) {
                    $texte = substr($texte, 0, $p) . $code . substr($texte, $p);
            } else {
                    $texte .= $code;
            }
        }
    }
    return $texte;
}

function ciparam_jquery_plugins($plugins) {
    $diff = array();
    
    if (is_array($plugins) 
            AND defined('_CIPARAM_SITE_PUBLIC_EN_JQUERY_1_7_2') 
            AND _CIPARAM_SITE_PUBLIC_EN_JQUERY_1_7_2=='oui'){
        
	$liste_spip32 =	array(
            'javascript/jquery.js',
            'javascript/jquery-migrate-3.0.1.js',
            'javascript/jquery.form.js',
            'javascript/jquery.autosave.js',
            'javascript/jquery.placeholder-label.js',
            'javascript/ajaxCallback.js',
            'javascript/js.cookie.js',
            'javascript/jquery.cookie.js'
            );
        
	$liste_v172 =	array(
            '_js/v172/jquery.js',
            '_js/v172/jquery.form.js',
            '_js/v172/jquery.autosave.js',
            '_js/v172/jquery.placeholder-label.js',
            '_js/v172/ajaxCallback.js',
            '_js/v172/jquery.cookie.js'
            );
        
        $diff = array_diff($plugins,$liste_spip32);
        $plugins = array_merge($diff, $liste_v172);
    }    
    return $plugins;
}

// Mettre le tri par défaut
// lors de la création d'une rubrique
function ciparam_pre_insertion($flux) {
    if ($flux['args']['table'] == 'spip_rubriques') {
        if (empty($flux['data']['citri']) 
            AND empty($flux['data']['citrinum']) 
            AND empty($flux['data']['citriinverse'])){

                include_spip('inc/ciparam_inc_commun');
                $citridefaut = ciparam_tri_par_defaut("spip_rubriques");
                $citrirow = ciparam_tri_to_row($citridefaut['nom'],$citridefaut['ordre']);
                $flux['data']['citri'] = $citrirow['tri'];
                $flux['data']['citrinum'] = $citrirow['trinum'];
                $flux['data']['citriinverse'] = $citrirow['triinverse'];
        }
    }

    return $flux;
}

?>