<?php
/**
 * Plugin Previsualisation etendue
 * Copyright (c) Christophe IMBERTI
 * Licence Creative commons by-nc-sa
 */



/**
 * Mecanisme de previsualisation propre au plugin
 * qui permet de beneficier du cache de premier niveau de SPIP 
 * (cette fonction intervient lors du calcul de la page)
 *
 * @param 	$objet,$m_statut,$m_id_rubrique
 * @return 	tableau pour la clause where
 */
function cipr_where($objet,$m_statut,$m_id_rubrique){
	$return = array("=", "$m_statut", "'publie'");

	// Cas ou l'on est authentifie
	if (isset($GLOBALS['visiteur_session']['statut'])) {
		// Utiliser $_GET['var_mode']
//		if ($_GET['var_mode']=='cipreview') {
		if ($_GET['cipreview']=='non') {
			// ne rien faire dans ce cas
		} elseif ($_GET['var_mode']=='preview' OR $_GET['var_mode']=='cipreview' OR isset($_COOKIE['cipr_preview'])) {
			// Autorisation de previsualiser
			if (!empty($GLOBALS['meta']['preview']) AND strpos($GLOBALS['meta']['preview'], ",". $GLOBALS['visiteur_session']['statut'] .",")!==false) {
				
				$publie_ou_prop_ou_en_cours_redaction = "('publie','prop','prepa')";
				$publie_ou_prop = "('publie','prop')";
				$en_cours_redaction = "('prepa')";
				
				switch ($objet){
					case 'hierarchie':
					case 'rubriques':
						// pouvoir previsualiser une rubrique non publiee 
						// 'new' et '0' sont indispensables
						$publie_ou_prop_ou_en_cours_redaction = "('publie','prive','new','0')";
						$publie_ou_prop = "('publie')";
						$en_cours_redaction = "('prive','new','0')";
						break;					
				}
				
				$return = array("IN", "$m_statut", $publie_ou_prop_ou_en_cours_redaction);

				// auteur restreint a certaines rubriques
				// verifier que les elements en cours de redaction
				// sont dans une rubrique affectee a l'auteur
				$liste_rubriques = cipr_liste_rubriques_auteur();
				if ($liste_rubriques) {
					// la notion "en cours de redaction" ne s'applique pas aux breves ni aux sites references
					if (in_array($objet, array('hierarchie','rubriques','articles'))){
						$return = array("OR", 
									array("IN", "$m_statut", $publie_ou_prop),
									array("AND",
										array("IN", "$m_statut", $en_cours_redaction),
										sql_in($m_id_rubrique, $liste_rubriques)
										)
									);
					}
				}
				
			}
		}
	}

	return $return;

}


/**
 * Mecanisme de previsualisation propre au plugin
 * qui permet de beneficier du cache de premier niveau de SPIP 
 * (cette fonction intervient lors du calcul de la page)
 * Ce fonction est utile pour la boucle document qui est tres particuliere 
 *
 * @param 	$objet
 * @return 	liste des statuts
 */
function cipr_where_document($objet){
	$return = "('publie')";

	// Cas ou l'on est authentifie
	if (isset($GLOBALS['visiteur_session']['statut'])) {
		// Utiliser $_GET['var_mode']
//		if ($_GET['var_mode']=='cipreview') {
		if ($_GET['cipreview']=='non') {
			// ne rien faire dans ce cas
		} elseif ($_GET['var_mode']=='preview' OR $_GET['var_mode']=='cipreview' OR isset($_COOKIE['cipr_preview'])) {
			// Autorisation de previsualiser
			if (!empty($GLOBALS['meta']['preview']) AND strpos($GLOBALS['meta']['preview'], ",". $GLOBALS['visiteur_session']['statut'] .",")!==false) {
				
				$publie_ou_prop_ou_en_cours_redaction = "('publie','prop','prepa')";
				
				switch ($objet){
					case 'hierarchie':
					case 'rubriques':
						// pouvoir previsualiser une rubrique non publiee 
						// 'new' et '0' sont indispensables
						$publie_ou_prop_ou_en_cours_redaction = "('publie','prive','new','0')";
						break;					
				}
				
				$return = $publie_ou_prop_ou_en_cours_redaction;
			}
		}
	}

	return $return;

}


/**
 * Renvoie, sous forme de liste (et pas sous forme de tableau),
 * les rubriques liees a cet auteur, independamment de son statut
 *
 * @param 	
 * @return 	$liste_rubriques
 */
function cipr_liste_rubriques_auteur() {
	static $liste_rubriques;

	if (!$liste_rubriques) {
		include_spip('inc/autoriser');
		$liste_rubriques = implode(",",liste_rubriques_auteur($GLOBALS['visiteur_session']['id_auteur']));

		if (!$liste_rubriques)
			$liste_rubriques = "0";
	}
    return $liste_rubriques;
}


/**
 * Mecanisme pour les article post date
 * (cette fonction intervient lors du calcul de la page)
 *
 * @param 	$objet,$m_statut,$m_date
 * @return 	tableau pour la clause where
 */
function cipr_postdates(){

	if ($GLOBALS['meta']['date_prochain_postdate'] > time())
		$return = date('Y-m-d H:i:s', $GLOBALS['meta']['date_prochain_postdate']);
	else
		$return = '2138-01-01 00:00:00';
	
	// Cas ou l'on est authentifie
	if (isset($GLOBALS['visiteur_session']['statut'])) {
		if ($_GET['cipreview']=='non') {
			// ne rien faire dans ce cas
		} elseif ($_GET['var_mode']=='preview' OR $_GET['var_mode']=='cipreview' OR isset($_COOKIE['cipr_preview'])) {
			// Autorisation de previsualiser
			if (!empty($GLOBALS['meta']['preview']) AND strpos($GLOBALS['meta']['preview'], ",". $GLOBALS['visiteur_session']['statut'] .",")!==false)
				$return = '2138-01-01 00:00:00';
		}
	}

	return $return;
}

?>