//----------------------------------------------------
// Plugin CIWIDGET - widget graphique
//----------------------------------------------------

function ciwidget_graphique_affiche_donnees(chart,donnees,type,serie_colonne,entete_colonne,entete_ligne,tableau_options_des_series,opacite,radius,moyenne_active,moyenne_ordre){
  chart.data.labels = ciwidget_graphique_labels(chart,donnees,type,serie_colonne,entete_colonne,entete_ligne);
  chart.data.datasets = ciwidget_graphique_datasets(donnees,serie_colonne,entete_colonne,entete_ligne,type);
  if (type=='bar' && moyenne_active=='oui'){
    ciwidget_graphique_moyenne_mobile(chart,type,moyenne_ordre);
  }
  
  ciwidget_graphique_couleurs(chart,type,tableau_options_des_series,opacite,radius);
  chart.update();
}


function ciwidget_graphique_moyenne_mobile(chart,type,moyenne_ordre){
    var serie = 0;
    var nb_series = chart.data.datasets.length;
    var longueur = 0;
    var tableau = [];
    var i = 0;
    var valeur = 0;
    var prendre_le_max = true;
    
    if(typeof chart.options.scales.yAxes[0].stacked !== 'undefined' 
            && chart.options.scales.yAxes[0].stacked===true){
        prendre_le_max = false;        
    }

    if (nb_series>0){
        if (nb_series==1){
            // une seule serie
            tableau = chart.data.datasets[0].data;
            chart.data.datasets[0].type = type;
            
        } else {
            // si plusieurs series, les cumuler ou bien prendre la valeur maximum
            longueur = chart.data.datasets[0].data.length;
            
            for (i=0; i < longueur; i++){
                valeur = 0;
                for (serie = 0; serie < nb_series; serie++) {
                    if (chart.data.datasets[serie].data[i] != "null") {
                        if (prendre_le_max){
                            if (parseFloat(chart.data.datasets[serie].data[i]) > valeur){
                                valeur = parseFloat(chart.data.datasets[serie].data[i]);
                            }
                        } else {
                            valeur += parseFloat(chart.data.datasets[serie].data[i]);
                        }
                    }
                }
                tableau.push(valeur);
            }
                
            for (serie = 0; serie < nb_series; serie++) {
                chart.data.datasets[serie].type = type;
            }
        }
    
        if (tableau.length > moyenne_ordre){

            chart.data.datasets.push({
            type: "line",
            label: "moyenne_mobile",
            fill: false,
            pointRadius: 0,
            borderWidth: 4,
            data: []
            });

            chart.data.datasets[nb_series].data = ciwidget_graphique_moyenne_mobile_calcul(tableau, moyenne_ordre);
        }
    }
}

/**
    * Calcul de la moyenne mobile
    * @param tableau - tableau de la serie de donnees
    * @param moyenne_ordre - nombre d'elements a inclure dans le calcul (cf. definition INSEE)
    */
function ciwidget_graphique_moyenne_mobile_calcul(tableau, moyenne_ordre){
    var i = 0;
    var rayon = (moyenne_ordre - 1) / 2;
    var longueur = tableau.length;
    var resultat = [];
    var debut = 0;
    var fin = 0;

    // calcul d'une moyenne
    var avg = function(array){
        var moyenne = null;
        var sum = 0;
        var count = 0;
        var i = 0;
        
        for (i=0; i < array.length; i++){
            sum += parseFloat(array[i]);
            count++;
        }

        if (count>0){
            moyenne = sum / count;
        }
        
        return moyenne;
    };


    // calcul de la moyenne mobile
    for (i=0; i < longueur; i++){
        
        if (i >= rayon){
            debut = i - rayon;
        } else {
            debut = 0;
        }
        if (i <= longueur - rayon){
            fin = i + rayon;
        } else {
            fin = longueur;
        }

        resultat.push(avg(tableau.slice(debut, fin)));
    }

    return resultat;
}

function ciwidget_hex2rgb(hex,opacity=1) {
    if(typeof hex === 'undefined'){
        return '';
    } else {
        hex   = hex.replace('#', '');
        var r = parseInt(hex.length == 3 ? hex.slice(0, 1).repeat(2) : hex.slice(0, 2), 16);
        var g = parseInt(hex.length == 3 ? hex.slice(1, 2).repeat(2) : hex.slice(2, 4), 16);
        var b = parseInt(hex.length == 3 ? hex.slice(2, 3).repeat(2) : hex.slice(4, 6), 16);

        return "rgb("+ r + "," + g + "," + b + ", "+opacity+")";
    }
}

function ciwidget_verifier_hex(hex){
    return /^#([0-9A-F]{3}){1,2}$/i.test(hex);
}

function ciwidget_graphique_labels(chart,donnees,type,serie_colonne,entete_colonne,entete_ligne){
    var tableau = [];
    var colonne_debut = 0;
    var ligne_debut = 0;
    var rotation_auto = false;
    if(typeof chart.options.scales !== 'undefined'){
        if(typeof chart.options.scales.xAxes[0].ticks.maxRotation !== 'undefined'){
            if (chart.options.scales.xAxes[0].ticks.maxRotation>0){
                rotation_auto = true;
            }
        } 
    }
    
    if(typeof donnees[0] !== 'undefined'){ 

        if(serie_colonne == "oui"){
            // Si les series sont en colonnes, les labels sont les en tetes de ligne
            if (entete_ligne=='oui'){
                
                if (entete_colonne=='oui'){
                    ligne_debut = 1;
                }
                
                for(i = ligne_debut; i < donnees.length; i++){ 
                    // eviter les lignes vides
                    if(typeof donnees[i][0] !== 'undefined' && donnees[i][0] != null && donnees[i][0] != ''){
                        if (rotation_auto){
                           tableau.push(donnees[i][0]); 
                        } else {
                            if (type == "pie" || type == "doughnut"){ 
                               tableau.push(donnees[i][0]); 
                            } else {
                               tableau.push(ciwidget_graphique_label_multilignes(donnees[i][0])); 
                           }
                        }
                    } else {
                       break;
                    }
                }
            }
        } else {
            // Si les series sont en lignes, les labels sont les en tetes de colonne
            if (entete_colonne=='oui'){
                
                if (entete_ligne=='oui'){
                    colonne_debut = 1;
                }
                
                for(var i = colonne_debut; i < donnees[0].length; i++){ 
                    // eviter les colonnes vides
                    if(typeof donnees[0][i] !== 'undefined' && donnees[0][i] != null && donnees[0][i] != ''){
                        if (rotation_auto){
                            tableau.push(donnees[0][i]);
                        } else {
                            if (type == "pie" || type == "doughnut"){ 
                                tableau.push(donnees[0][i]);
                            } else {
                                tableau.push(ciwidget_graphique_label_multilignes(donnees[0][i]));
                            }
                        }
                    } else {
                        break;
                    }
                }
            }
        }
    }
    return tableau;
}

function ciwidget_graphique_label_multilignes(str){
    var sections = [];
    var words = str.split(" ");
    var temp = "";
    // afficher un label sur plusieurs lignes s'il depasse le nombre de caracteres max 
    var maxwidth = 10;

    words.forEach(function(item, index){
        if(temp.length > 0){
            var concat = temp + ' ' + item;

            if(concat.length > maxwidth){
                sections.push(temp);
                temp = "";
            } else {
                if(index == (words.length-1)){
                    sections.push(concat);
                    return;
                } else {
                    temp = concat;
                    return;
                }
            }
        }

        if(index == (words.length-1)){
            sections.push(item);
            return;
        }

        if(item.length < maxwidth){
            temp = item;
        } else {
            sections.push(item);
        }

    });

    return sections;
}

function ciwidget_graphique_datasets(donnees,serie_colonne,entete_colonne,entete_ligne,type){
    var tableau = []
    var colonne_debut = 0;
    var ligne_debut = 0;
    var nombre_colonnes = 0;
    var ci_index = 0;
    var temp = [];
    
    // Ecarter les en tetes de colonne ou de ligne
    if (entete_colonne == 'oui'){
        ligne_debut = 1;
    }
    if (entete_ligne == 'oui'){
        colonne_debut = 1;
    }

    // series en colonnes
    if(serie_colonne == "oui"){
        nombre_colonnes = donnees[0].length;
        if(entete_colonne == 'oui'){ 
            ci_index = 0;
            // Parcours de la premiere ligne
            for(var i = colonne_debut; i < donnees[0].length; i++){ 
                if (typeof donnees[0][i] !== "undefined"){
                    if(donnees[0][i] != null){
                        tableau[ci_index] = {};
                        tableau[ci_index].label = donnees[0][i];
                        tableau[ci_index].data = [];
                        ci_index++;
                    } else { 
                        break;
                    }
                }    
            }
        }

        
        // parcours de toutes les lignes (sauf en tete)
        for(var i = ligne_debut; i < donnees.length; i++){
            ci_index = 0;

            if (typeof donnees[i][colonne_debut] !== "undefined"){

                // parcours de toutes les colonnes (sauf en tete)
                for(var j = colonne_debut; j <= nombre_colonnes; j++){
                    if (donnees[i][j] != null){
                        // pour une cellule vide, le graphique veut "null", mais dans le tableau l'utilisateur ne mettra pas "null"
                        if (donnees[i][j] == ''){
                            donnees[i][j] = "null";
                        } 
                        if (typeof tableau[ci_index]  === "undefined"){
                            tableau[ci_index] = {data : []};
                        }
                        tableau[ci_index].data.push(donnees[i][j]);
                        ci_index++;
                    } else { 
                        break;
                    }
                }
            } else {
                if (typeof tableau[ci_index]  === "undefined"){
                    tableau[ci_index] = {data : []};
                }
                tableau[ci_index].data.push(donnees[i]);
                ci_index++;
            }
        }
        
    // series en lignes
    } else {
        
        for(var i = ligne_debut; i < donnees.length; i++){
            if(typeof donnees[i] !== 'undefined' && typeof donnees[i][0] !== 'undefined' && donnees[i][0] != null){
                tableau[ci_index] = {};
                temp = [];
                if(entete_ligne == 'oui'){
                    tableau[ci_index].label = donnees[i][0];
                }
                
                if (typeof donnees[i][colonne_debut] !== "undefined"){
                    for(var j = colonne_debut; j < donnees[i].length; j++){
                        if(donnees[i][j] != null){ 
                           // pour une cellule vide, le graphique veut "null", mais dans le tableau l'utilisateur ne mettra pas "null"
                            if (donnees[i][j]==''){
                                donnees[i][j] = "null";
                            } 
                            temp.push(donnees[i][j]);
                        }
                    }
                } else {
                    temp.push(donnees[i]);
                }
                tableau[ci_index].data = temp;
                ci_index++;
            } else { 
                break;
            }
        }
    }
  
    return tableau;
}


function ciwidget_graphique_couleurs(chart,type,tableau_options_des_series,opacite,radius){

    if (type == "pie" || type == "doughnut"){ 
        if(typeof chart.data.datasets[0].backgroundColor === 'undefined'){
            chart.data.datasets[0].backgroundColor = [];
        }
        for(var i = 0; i < chart.data.datasets[0].data.length; i++){
            if(typeof tableau_options_des_series['couleurs'][i] !== 'undefined' && ciwidget_verifier_hex(tableau_options_des_series['couleurs'][i])){
                chart.data.datasets[0].backgroundColor[i] = ciwidget_hex2rgb(tableau_options_des_series['couleurs'][i], opacite);
            }
        }
        
    } else {
        for(var i = 0; i < chart.data.datasets.length; i++){
            if(typeof tableau_options_des_series['couleurs'] !== 'undefined'){
                if(typeof tableau_options_des_series['couleurs'][i] !== 'undefined' && ciwidget_verifier_hex(tableau_options_des_series['couleurs'][i])){
                    chart.data.datasets[i].backgroundColor = ciwidget_hex2rgb(tableau_options_des_series['couleurs'][i], opacite);
                    chart.data.datasets[i].borderColor = ciwidget_hex2rgb(tableau_options_des_series['couleurs'][i], 1);
                }
            }
            
            if(typeof tableau_options_des_series['points'] !== 'undefined'){
                if(typeof tableau_options_des_series['points'][i] !== 'undefined'){

                    if (tableau_options_des_series['points'][i]!='aucun_point'){
                        chart.data.datasets[i].pointStyle = tableau_options_des_series['points'][i];
                        chart.data.datasets[i].pointRadius = tableau_options_des_series['radius'][i];
                        chart.data.datasets[i].pointHoverRadius = tableau_options_des_series['radius'][i];
                    } else {
                        chart.data.datasets[i].pointStyle = 'circle';
                        chart.data.datasets[i].pointRadius = 0;
                        chart.data.datasets[i].pointHoverRadius = 0;
                    }

                    chart.data.datasets[i].pointBackgroundColor = ciwidget_hex2rgb(tableau_options_des_series['couleurs'][i],1);
                    chart.data.datasets[i].pointBorderWidth = 1;
                }
            }
            
            if(typeof tableau_options_des_series['showline'] !== 'undefined'){
                if(typeof tableau_options_des_series['showline'][i] !== 'undefined'){
                    if (tableau_options_des_series['showline'][i]=='non'){
                        chart.data.datasets[i].showLine = false; 
                    } else if (tableau_options_des_series['showline'][i]=='oui'){
                        chart.data.datasets[i].showLine = true; 
                    }
                }
            }
            
            if(typeof tableau_options_des_series['lineborderwidth'] !== 'undefined'){
                if(typeof tableau_options_des_series['lineborderwidth'][i] !== 'undefined'){
                    if (parseInt(tableau_options_des_series['lineborderwidth'][i])>0){
                        chart.data.datasets[i].borderWidth = parseInt(tableau_options_des_series['lineborderwidth'][i]); 
                    }
                }
            }
            
            if(typeof tableau_options_des_series['lineborderdash'] !== 'undefined'){
                if(typeof tableau_options_des_series['lineborderdash'][i] !== 'undefined'){
                    if (parseInt(tableau_options_des_series['lineborderdash'][i])>0){
                        chart.data.datasets[i].borderDash = [parseInt(tableau_options_des_series['lineborderdash'][i]),parseInt(tableau_options_des_series['lineborderdash'][i])]; 
                    }
                }
            }
        }
    }
}

function ciwidget_separer_millier(nombre) {
    var texte, entier, fraction, decIndex;
    
    if (nombre){
        texte = nombre.toString();
        decIndex = texte.lastIndexOf('.');
        if (decIndex > 0) {
            entier = texte.substr(0, decIndex);
            fraction = texte.substr(decIndex);
        } else {
            entier = texte;
        }
        var rgx = /(\d+)(\d{3})/;
        while (rgx.test(entier)) {
            entier = entier.replace(rgx, '$1' + ' ' + '$2');
        }
        return fraction ? entier + fraction : entier;
    } else {
        return nombre;
    }    
}

function ciwidget_multilignes(nom) {
    var taille = 0;
    var phrase = "";
    var seuil = 70;
    var tableau_final = [];
    
    if (nom.length>80){
        var tableau = nom.split(" ");
        for(var i = 0; i < tableau.length; i++) {
            taille = taille + tableau[i].length;

            if(taille > seuil) {
                tableau_final.push(phrase);
                phrase = "";
                taille = 0;
            }
            
            phrase += tableau[i] + " ";
        }
        
        if (phrase){
            tableau_final.push(phrase);
        }
        
        return tableau_final;
    } else {
        return nom;
    }
}
