<?php
/**
 * Plugin CIWIDGET
 * Copyright (c) Christophe IMBERTI
 * Licence Creative commons by-nc-sa
 */

if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

include_spip('inc/widget_commun');
include_spip('inc/ciwidget_verifier');
include_spip('inc/filtres_boites');


function formulaires_ciwidget_iframe_charger_dist($objet = '', $id_objet = '', $id_widget = 0, $retour = '') {
	if (!autoriser('modifier',$objet,$id_objet))
		return false;

        $valeurs = array('titre'=>'',
            'descriptif'=>'',
            'code'=>'',
            'largeur'=>ciwidget_lire_meta_cle('iframe','largeur_defaut'),
            'hauteur'=>ciwidget_lire_meta_cle('iframe','hauteur_defaut'),
            'affichertitreetdesc'=>'oui',
            'conserverleratio'=>'non',
            'autoriserfullscreen'=>'oui');
        
        $id_widget = intval($id_widget);
	if ($id_widget>0) {
		$result = sql_select("*", "spip_ciwidget", "id_widget=$id_widget");
		while ($row = sql_fetch($result)) { 
			$valeurs = $row;
		}
        
                // anti hack
                if (isset($valeurs['type_widget']) AND $valeurs['type_widget']!='iframe'){
                    return false;
                }
                
                // unserialize le champ contenu
                if (isset($valeurs['contenu'])){
                    $contenu = unserialize($valeurs['contenu']);
                    $valeurs['contenu'] = '';
                    if ($contenu AND is_array($contenu)){
                        $valeurs = array_merge($valeurs,$contenu);
                    }
                }
	}
        
        $valeurs['retour_objet'] = ciwidget_retour_vers_objet($objet, $id_objet);

	return $valeurs;
}

function formulaires_ciwidget_iframe_verifier_dist($objet = '', $id_objet = '', $id_widget = 0, $retour = '') {
        $erreurs = array();
        
	$titre = _request('titre');	
	if (!$titre){
		$erreurs['titre'] = _T('info_obligatoire');
        }

	$largeur = _request('largeur');	
	if (!$largeur){
		$erreurs['largeur'] = _T('info_obligatoire');
        } else {
            if (!ctype_digit($largeur)){
		$erreurs['largeur'] = _T('ciwidget:valeur_incorrecte');
            } elseif (intval($largeur)<intval(ciwidget_lire_meta_cle('commun','largeur_min'))){
		$erreurs['largeur'] = _T('ciwidget:valeur_incorrecte');
            } elseif (intval($largeur)>intval(ciwidget_lire_meta_cle('commun','largeur_max'))){
		$erreurs['largeur'] = _T('ciwidget:valeur_incorrecte');
            }
        }
        
	$hauteur = _request('hauteur');	
	if (!$hauteur){
		$erreurs['hauteur'] = _T('info_obligatoire');
        } else {
            if (!ctype_digit($hauteur)){
		$erreurs['hauteur'] = _T('ciwidget:valeur_incorrecte');
            } elseif (intval($hauteur)<intval(ciwidget_lire_meta_cle('commun','hauteur_min'))){
		$erreurs['hauteur'] = _T('ciwidget:valeur_incorrecte');
            } elseif (intval($hauteur)>10000){
		$erreurs['hauteur'] = _T('ciwidget:valeur_incorrecte');
            }
        }
        
	$code = _request('code');	
	if (!$code){
		$erreurs['code'] = _T('info_obligatoire');
        } else {
                $url = ciwidget_extraire_src_iframe_ou_url($code);
                $embed_code = ciwidget_embed_iframe($code);
                
                if (!ciwidget_verifier_url_absolue_ou_relative($url)){
        		$erreurs['code'] = _T('ciwidget:valeur_incorrecte');
                        
                // verifier par rapport a la liste blanche
                } elseif (ciwidget_lire_meta_cle('iframe','limiter_hosts')=='oui' 
                    AND !ciwidget_verifier_url_dans_liste_blanche($url,ciwidget_lire_meta_cle('iframe','hosts_liste'))){
        		$erreurs['code'] = _T('ciwidget:host_interdit');
                        
                // verifier par rapport aux widgets dedies
                } elseif ($widget_dedie = ciwidget_url_identifier_widget_dedie($url)) {
        		$erreurs['code'] = _T('ciwidget:host_widget_'.$widget_dedie);
                } elseif ($embed_code){
                    if (!ciwidget_embed_iframe_html_correct($code)){
        		$erreurs['code'] = _T('ciwidget:valeur_incorrecte');
                    }
                }
        }
        
	$affichertitreetdesc = _request('affichertitreetdesc');	
        if (!ciwidget_verifier_oui_non($affichertitreetdesc)){
                $erreurs['affichertitreetdesc'] = _T('ciwidget:valeur_incorrecte');
        }
        
	$conserverleratio = _request('conserverleratio');	
        if (!ciwidget_verifier_oui_non($conserverleratio)){
                $erreurs['conserverleratio'] = _T('ciwidget:valeur_incorrecte');
        }
        
	$autoriserfullscreen = _request('autoriserfullscreen');	
        if (!ciwidget_verifier_oui_non($autoriserfullscreen)){
                $erreurs['autoriserfullscreen'] = _T('ciwidget:valeur_incorrecte');
        }
        
	return $erreurs;
}

function formulaires_ciwidget_iframe_traiter_dist($objet = '', $id_objet = '', $id_widget = 0, $retour = '') {
        $id_widget = intval($id_widget);

        $c = array();
	foreach (array('titre', 'descriptif') as $champ){
		$c[$champ] = _request($champ);
        }
        
        $code = _request('code');
        $embed_code = ciwidget_embed_iframe($code);
        $embed = 'non';
        if ($embed_code){
            $code = $embed_code;
            $embed = 'oui';
        }
        
        $affichertitreetdesc = _request('affichertitreetdesc');
        $conserverleratio = _request('conserverleratio');
        $autoriserfullscreen = _request('autoriserfullscreen');
        
        $largeur = ciwidget_verif_largeur_sinon_defaut(_request('largeur'), ciwidget_lire_meta_cle('iframe','largeur_defaut'));
        
        $hauteur_int = intval(_request('hauteur'));	
        $hauteur_min = intval(ciwidget_lire_meta_cle('commun','hauteur_min'));
        if ($hauteur_int>0){
            if ($hauteur_int<$hauteur_min) {
                $hauteur_int = $hauteur_min;
            }
        } else {
                $hauteur_int = intval(ciwidget_lire_meta_cle('iframe','hauteur_defaut'));
        }
        $hauteur = strval($hauteur_int);
        
        $type_widget = 'iframe';
        $c['contenu'] = serialize(array('code'=>$code, 
            'largeur'=>$largeur, 
            'hauteur'=>$hauteur, 
            'affichertitreetdesc'=>$affichertitreetdesc, 
            'conserverleratio'=>$conserverleratio, 
            'autoriserfullscreen'=>$autoriserfullscreen, 
            'embed'=>$embed));
        
        $c['type_widget'] = $type_widget;
        $c['objet'] = $objet;
        $c['id_objet'] = $id_objet;

	if ($id_widget>0) {
            // modifier
            sql_updateq("spip_ciwidget", $c, "id_widget=$id_widget");
        } else {
            // ajouter et lier
            $id_widget = sql_insertq('spip_ciwidget', $c);
        }
    
        $res = ciwidget_retour_formulaire($objet,$id_objet,$type_widget,$retour);

	return $res;
}

function ciwidget_embed_iframe($code=''){
    $return = '';
    $tableau_style = array();
    $ok = false;
    
    // au moins une balise iframe et une balise div
    if (stripos($code, '<iframe') !== false 
            AND stripos($code, '<div') !== false){

        // au moins un style avec un padding en %
        if (preg_match_all('/style="(.*?)"/is', str_replace("'",'"',$code), $styles)){
            foreach ($styles[1] as $style) {
                $tableau_style = explode(';',$style);
                foreach ($tableau_style as $tableau_style_element) {
                    if (substr(strtolower(trim($tableau_style_element)),0,7)=='padding'){
                        if (strpos($tableau_style_element,'%')!==false){
                            $ok = true;
                            break;
                        }
                    }
                }
                if ($ok){
                    break;
                }
            }
        }
        
        if ($ok){
            // securite
            $return = interdire_scripts($code);
        }
        
    }
    
    return $return;
}

function ciwidget_embed_iframe_html_correct($code=''){
    $return = true;
    
    $code_a_verifier = strtolower($code);
    $code_a_verifier = str_replace('&lt;','<',$code_a_verifier);

    // verifier que chaque balise ouvrante a une balise fermante
    if (substr_count($code_a_verifier,'<iframe') != substr_count($code_a_verifier,'</iframe>')){
        $return = false;
    }
    if (substr_count($code_a_verifier,'<div') != substr_count($code_a_verifier,'</div>')){
        $return = false;
    }
    // au maximum un seul iframe
    if (substr_count($code_a_verifier,'<iframe') > 1){
        $return = false;
    }
    // au maximum un seul div
    if (substr_count($code_a_verifier,'<div') > 1){
        $return = false;
    }
    // le div doit etre avant l'iframe
    if (strpos($code_a_verifier,'<div') > strpos($code_a_verifier,'<iframe')){
        $return = false;
    }
    
    return $return;
}

?>