<?php
/**
 * Plugin CIWIDGET
 * Copyright (c) Christophe IMBERTI
 * Licence Creative commons by-nc-sa
 */

if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

include_spip('inc/ciwidget_inc_meta');
include_spip('inc/ciwidget_commun');
include_spip('inc/widget_commun');

function ciwidget_document_charger($objet='', $id_objet='', $type_widget='', $id_widget=0, $retour='', $champs_supplementaires_defaut=array(), $champs_supplementaires_renseignes=array()){
    
    	if ($objet AND $id_objet AND !$id_widget){
            $valeurs = formulaires_joindre_document_charger_dist('new', $id_objet, $objet);
            $valeurs['id_widget'] = 'new';
            $valeurs['retour_objet'] = ciwidget_retour_vers_objet($objet, $id_objet, $retour);
            return $valeurs;
        }
        
        $largeur_defaut = 0;
        $ci_largeur_defaut = ciwidget_lire_meta_cle($type_widget,'largeur_defaut');
        if (intval($ci_largeur_defaut)>0){
            $largeur_defaut = strval(intval($ci_largeur_defaut));
        }
        
        $hauteur_defaut = 0;
        $ci_hauteur_defaut = ciwidget_lire_meta_cle($type_widget,'hauteur_defaut');
        if (intval($ci_hauteur_defaut)>0){
            $hauteur_defaut = strval(intval($ci_hauteur_defaut));
        }
        
        $valeurs = array(
            'titre'=>'',
            'descriptif'=>'',
            'duree_editable'=>'oui',
            'credits'=>'',
            'fichier'=>'',
            'url'=>'',
            'id_document'=>'',
            'vignette'=>'',
            'id_vignette'=>'',
            'duree'=>'00:00:00',
            'largeur'=>$largeur_defaut,
            'hauteur'=>$hauteur_defaut
             );
        
        $valeurs = array_merge($valeurs,$champs_supplementaires_defaut);
        
        
        $id_widget = intval($id_widget);
	if ($id_widget>0) {
                $valeurs['id_widget'] = $id_widget;
            
		$result = sql_select("*", "spip_ciwidget", "id_widget=$id_widget");
		while ($row = sql_fetch($result)) { 
			$valeurs = $row;
		}
        
                // anti hack
                if (isset($valeurs['type_widget']) AND $valeurs['type_widget']!=$type_widget){
                    return false;
                }
                
                // unserialize le champ contenu
                if (isset($valeurs['contenu'])){
                    $contenu = unserialize($valeurs['contenu']);
                    $valeurs['contenu'] = '';
                    if ($contenu AND is_array($contenu)){
                        $valeurs = array_merge($valeurs,$contenu);
                    }
                }
                
                if (isset($valeurs['id_document'])){
                    $id_document = intval($valeurs['id_document']);
                    $doc = sql_fetsel('*', 'spip_documents', 'id_document=' . $id_document);
                    if ($doc){
                        $valeurs['descriptif'] = $doc['descriptif'];
                        $valeurs['credits'] = $doc['credits'];
                        if ($doc['largeur']>0){
                            $valeurs['largeur'] = strval($doc['largeur']);
                        }
                        if ($doc['hauteur']>0){
                            $valeurs['hauteur'] = strval($doc['hauteur']);
                        }
                        $valeurs['duree'] = date('H:i:s', mktime(0, 0, intval($doc['duree'])));
                        $valeurs['fichier'] = $doc['fichier'];
                        $valeurs['taille'] = $doc['taille'];
                        
                        // type du document et inclusion
                        $row2 = sql_fetsel(
                                'titre as type_document,inclus',
                                'spip_types_documents',
                                'extension=' . sql_quote($doc['extension'])
                        );
                        if ($row2){
                            $valeurs['type_document'] = $row2['type_document'];
                        }
                    }
                }
	}

        // enlever le dossier pour l'affichage du fichier
        // (mais pas pour une URL)
        if (isset($valeurs['fichier']) AND $valeurs['fichier']){
            $pos = strrpos($valeurs['fichier'],'/');
            if ($pos>0){
                if (ctype_alnum(substr($valeurs['fichier'],0,$pos-1))) {
                    $valeurs['fichier'] = substr($valeurs['fichier'],$pos+1);
                }
            }
        }

        // faut-il imposer une largeur ? une hauteur ?
        $ci_largeur_non_modifiable = ciwidget_lire_meta_cle($type_widget,'largeur_non_modifiable');
        if ($ci_largeur_non_modifiable AND strtolower($ci_largeur_non_modifiable)==='oui'){
            $valeurs['ci_largeur_non_modifiable']='oui';
            $valeurs['largeur']=$largeur_defaut;
        }
        $ci_hauteur_non_modifiable = ciwidget_lire_meta_cle($type_widget,'hauteur_non_modifiable');
        if ($ci_hauteur_non_modifiable AND strtolower($ci_hauteur_non_modifiable)==='oui'){
            $valeurs['ci_hauteur_non_modifiable']='oui';
            $valeurs['hauteur']=$hauteur_defaut;
        }
        
        $valeurs['retour_objet'] = ciwidget_retour_vers_objet($objet, $id_objet, $retour);

	return $valeurs;
}

function ciwidget_document_verifier($objet='', $id_objet='', $type_widget='', $id_widget=0, $retour='', $champs_supplementaires_defaut=array(), $champs_supplementaires_renseignes=array()){
        $id_widget = intval($id_widget);        
        $id_document = 0;
        $erreurs = array();

        if ($id_widget){
            if ($contenu_actuel = sql_getfetsel("contenu", "spip_ciwidget", "id_widget=$id_widget")){
                $contenu_actuel_unserialize = unserialize($contenu_actuel);
                if (isset($contenu_actuel_unserialize['id_document'])){
                    $id_document = intval($contenu_actuel_unserialize['id_document']);
                }
            }
        }

        if ($id_document){
            $verifier = charger_fonction('verifier', 'formulaires/joindre_document');
            $erreurs = $verifier($id_document);
            
        }

	return $erreurs;
}

function ciwidget_document_traiter($objet='', $id_objet='', $type_widget='', $id_widget=0, $retour='', $champs_supplementaires_defaut=array(), $champs_supplementaires_renseignes=array()){
        $id_widget = intval($id_widget);        
        $id_document = 0;
        $duree_editable = 'oui';
        $c = array();

        if ($id_widget){
            if ($contenu_actuel = sql_getfetsel("contenu", "spip_ciwidget", "id_widget=$id_widget")){
                $contenu_actuel_unserialize = unserialize($contenu_actuel);
                if (isset($contenu_actuel_unserialize['id_document'])){
                    $id_document = intval($contenu_actuel_unserialize['id_document']);
                }
                if (isset($contenu_actuel_unserialize['duree_editable'])){
                    $duree_editable = $contenu_actuel_unserialize['duree_editable'];
                }
            }
        }

        // cas du changement d'un doc
    	if ($objet AND $id_objet AND $id_widget AND $id_document AND _request('joindre_upload')){
                $res = formulaires_joindre_document_traiter_dist($id_document, $id_objet, $objet);

                $doc = sql_fetsel('*', 'spip_documents', 'id_document=' . $id_document);
                if ($doc){
                        // IMPORTANT : ne pas lier le document à l'objet
                        sql_delete("spip_documents_liens", "id_document=".$id_document." AND objet='".$objet."' AND id_objet=".$id_objet);
                        
                        // lier le document au widget si le lien n'existe plus
                        if ($id_widget>0){
                            if (!sql_countsel("spip_documents_liens", "id_document=".$id_document." AND objet='ciwidget' AND id_objet=".$id_widget)>0){
                                sql_insertq('spip_documents_liens', array('id_document'=>$id_document,'objet'=>'ciwidget','id_objet'=>$id_widget,'vu'=>'oui'));
                            }
                        }
                }
        }
        
        // cas du changement d'un doc distant
    	elseif ($objet AND $id_objet AND $id_widget AND $id_document AND _request('url') AND _request('joindre_distant')){
                $res = formulaires_joindre_document_traiter_dist($id_document, $id_objet, $objet);

                $doc = sql_fetsel('*', 'spip_documents', 'id_document=' . $id_document);
                if ($doc){
                        // IMPORTANT : ne pas lier le document à l'objet
                        sql_delete("spip_documents_liens", "id_document=".$id_document." AND objet='".$objet."' AND id_objet=".$id_objet);
                }
        }
        
    	elseif ($objet AND $id_objet AND (!$id_widget OR _request('joindre_upload'))){
                $res = formulaires_joindre_document_traiter_dist($id_document, $id_objet, $objet);

                if (!$id_document){
                    if (isset($res['redirect'])){
                            // id du document
                            $id_document = intval(str_replace("#doc", "", $res['redirect']));
                    }
                }
                
                if ($id_document  AND !$id_widget){
                        $doc = sql_fetsel('*', 'spip_documents', 'id_document=' . $id_document);
                        if ($doc){
                                // IMPORTANT : ne pas lier le document à l'objet
                                sql_delete("spip_documents_liens", "id_document=".$id_document." AND objet='".$objet."' AND id_objet=".$id_objet);

                                // titre
                                if ($doc['titre']){
                                    $titre = $doc['titre'];
                                } else {
                                    // sinon on se base sur le nom du fichier
                                    $titre = substr($doc['fichier'],strrpos($doc['fichier'], "/")+1); // Enlever le chemin
                                    $titre = substr($titre,0, strrpos($titre, ".")); // Enlever l'extension du nom du fichier
                                    $titre = preg_replace(',[[:punct:][:space:]]+,u', ' ', $titre);
                                    $titre = preg_replace(',\.([^.]+)$,', '', $titre);
                                }
                                
                                // creer le widget (avec comme titre le nom du fichier du doc)
                                $c['type_widget'] = $type_widget;
                                $c['titre'] = $titre;
                                $c['objet'] = $objet;
                                $c['id_objet'] = $id_objet;
                                if ($doc['duree']>0){
                                    $ci_duree_editable = 'non';
                                } else {
                                    $ci_duree_editable = 'oui';
                                }
                                
                                if ($doc){
                                    $c['contenu'] = serialize(array_merge(array('id_document'=>$doc['id_document'], 'duree_editable'=>$ci_duree_editable), $champs_supplementaires_defaut));
                                }

                                $id_widget = sql_insertq('spip_ciwidget', $c);
                                if (intval($id_widget)>0){
                                    // lier le document au widget
                                    sql_insertq('spip_documents_liens', array('id_document'=>$id_document,'objet'=>'ciwidget','id_objet'=>$id_widget,'vu'=>'oui'));
                                }

                                // retour
                                $libelle = _T('info_modification_enregistree');
                                $load = '<script type="text/javascript">ciwidget_callback_widget("'.$type_widget.'","'.$id_widget.'","'.$objet.'","'.$id_objet.'");</script>';
                                if ($retour) {
                                        // tient compte du cas sans popin et du cas d'une saisie via CISF
                                        $retour = ciwidget_url_widget_edit($objet,$id_objet,$type_widget,$id_widget,$retour);
                                        $res['redirect'] = $retour;
                                        $load = '';
                                } else {
                                        $res['editable'] = true;
                                }

                                $res['message_ok'] = $libelle . $load;

                                return $res;
                        }
                }
        }
    
        
        $c = array();
        $d = array();
	foreach (array('titre', 'descriptif') as $champ){
		$c[$champ] = _request($champ);
		$d[$champ] = $c[$champ];
        }
        
        if ($duree_editable=='oui'){
            $duree = 0;
            $request_duree = _request('duree');
            if (!empty($request_duree) AND preg_match('#([0-9]{1,2}):([0-9]{1,2}):([0-9]{1,2})#', $request_duree, $regs)) {
                $duree = (3600*intval($regs[1])) + (60*intval($regs[2])) + intval($regs[3]);
            }
        }

        if (_request('largeur')) {
            $largeur = ciwidget_verif_largeur_sinon_defaut(_request('largeur'), ciwidget_lire_meta_cle($type_widget,'largeur_defaut'));
            $d['largeur'] = $largeur;
        }
        if (_request('hauteur')) {
            $hauteur = ciwidget_verif_hauteur_sinon_defaut(_request('hauteur'), ciwidget_lire_meta_cle($type_widget,'hauteur_defaut'));
            $d['hauteur'] = $hauteur;
        }

        $c['contenu'] = serialize(array_merge(array('id_document'=>$id_document, 'duree_editable'=>$duree_editable), $champs_supplementaires_renseignes));
        $c['type_widget'] = $type_widget;
        $c['objet'] = $objet;
        $c['id_objet'] = $id_objet;
        
        if ($duree_editable=='oui'){
        	$d['duree'] = $duree;
        }
	$d['credits'] = _request('credits');
        
	if ($id_widget>0) {
            // modifier
            sql_updateq("spip_ciwidget", $c, "id_widget=$id_widget");
        } else {
            // ajouter
            $id_widget = sql_insertq('spip_ciwidget', $c);
        }
        
        // mettre à jour la table spip_documents
	if ($id_document){
            sql_updateq("spip_documents", $d, "id_document=$id_document");
        }
        
        $res = ciwidget_retour_formulaire($objet,$id_objet,$type_widget,$retour);
        
	return $res;
    
}

// Afficher le poids maximum pour l'upload
function ciwidget_upload_max_filesize($arg='') {
	$return = "";
	
        if (defined('_DIR_PLUGIN_BIGUP') AND ciwidget_utiliser_bigup()){
                include_spip('inc/config');
                $maxFileSize = intval(lire_config('bigup/max_file_size', 0));
                if ($maxFileSize > 0){
                        $return = $maxFileSize." Mo";
                }
        
        } elseif (function_exists('ini_get')) {
		if (($post_max = ini_get('post_max_size')) AND ($upload_max = ini_get('upload_max_filesize'))) {
			$max_upload_size = min(ciwidget_ini_to_num($post_max), ciwidget_ini_to_num($upload_max));
			if ($max_upload_size > 0){
				$return = ($max_upload_size/(1048576))." Mo";
                        }
		}
	}

        $return = pipeline('ciwidget_max_filesize', array('args'=>array(), 'data'=>$return));
        
	return $return;
}


// Transforme la notation de php.ini (comme '2M') en un entier (2*1048576)
function ciwidget_ini_to_num($valeur){ 
    $valeur = trim($valeur);
    $unite = substr($valeur, -1);
    $return = substr($valeur, 0, -1);
    $return = floatval($return);
    
    switch(strtoupper($unite)){
    case 'G':
        $return *= 1073741824;
        break;
    case 'M':
        $return *= 1048576;
        break;
    case 'K':
        $return *= 1024;
        break;
    }

    return $return;
}

function ciwidget_formats_image_autorises(){
    return array('jpg', 'png', 'gif');
}

function ciwidget_verifier_upload_formats_autorises($post,$type_format=''){
    $erreur = '';
    
    $files = array();
    $tableau_formats_autorises = array();
    
    if (is_array($post) AND in_array($type_format,array('image','video','audio'))) {
        if ($type_format == 'image'){
            $tableau_formats_autorises = ciwidget_formats_image_autorises();
        } elseif ($type_format == 'video'){
            $tableau_formats_autorises = ciwidget_tableau_formats_video_autorises();
        } elseif ($type_format == 'audio'){
            $tableau_formats_autorises = ciwidget_tableau_formats_audio_autorises();
        }
        
        $formats_autorises = implode(', ',$tableau_formats_autorises);
        
        include_spip('inc/renseigner_document');
        foreach ($post as $file) {
            if (is_array($file['name'])) {
                while (count($file['name'])) {
                    $test = array(
                        'error' => array_shift($file['error']),
                        'name' => array_shift($file['name']),
                        'tmp_name' => array_shift($file['tmp_name']),
                        );
                    if (!($test['error'] == 4)) {
                        if (preg_match(',\.([a-z0-9]+)(\?.*)?$,i', $test['name'], $match) and $fichier_tmp_ext = $match[1]) {
                            $fichier_tmp = $test['tmp_name'];

                            if (!in_array(strtolower($fichier_tmp_ext),$tableau_formats_autorises) ){
                                    $erreur = _T('ciwidget:erreur_format_'.$type_format, array('format_'.$type_format.'_autorise' => $formats_autorises));
                                    spip_unlink($fichier_tmp);

                            }
                        }
                    }
                }
            } else {
                if (!($file['error'] == 4)) {
                    if (preg_match(',\.([a-z0-9]+)(\?.*)?$,i', $file['name'], $match) and $fichier_tmp_ext = $match[1]) {
                        $fichier_tmp = $file['tmp_name'];

                        if (!in_array(strtolower($fichier_tmp_ext),$tableau_formats_autorises) ){
                                $erreur = _T('ciwidget:erreur_format_'.$type_format, array('format_'.$type_format.'_autorise' => $formats_autorises));
                                spip_unlink($fichier_tmp);

                        }
                    }
                }
            }
        }
    }
    
    return $erreur;        
}

function ciwidget_verifier_distant_formats_autorises($path='',$extension='',$type_format=''){
    $erreur = '';
    
    $tableau_formats_autorises = array();

    if ($path AND $extension AND in_array($type_format,array('image','video','audio'))) {
        if ($type_format == 'image'){
            $tableau_formats_autorises = ciwidget_formats_image_autorises();
        } elseif ($type_format == 'video'){
            $tableau_formats_autorises = ciwidget_tableau_formats_video_autorises();
        } elseif ($type_format == 'audio'){
            $tableau_formats_autorises = ciwidget_tableau_formats_audio_autorises();
        }
        
        $formats_autorises = implode(', ',$tableau_formats_autorises);
    
        if (!in_array(strtolower($extension),$tableau_formats_autorises) ){
                $erreur = _T('ciwidget:erreur_format_'.$type_format, array('format_'.$type_format.'_autorise' => $formats_autorises));
        }
    }
    
    return $erreur;        
}

function ciwidget_extension_fichier_distant($path){
    $extension = '';
    
    if ($path) {
        include_spip('inc/renseigner_document');
        $infos = renseigner_source_distante($path);
        $extension = '';
        if (isset($infos['extension'])){
            $extension = $infos['extension'];
        } elseif (isset($infos['mime_type'])){
            $tableau_mime = explode('/',$infos['mime_type']);
            if (isset($tableau_mime[1]) AND ctype_alnum($tableau_mime[1])){
                $extension = $tableau_mime[1];
            }
        } 
    }
    
    return $extension;        
}

function ciwidget_verifier_upload_debit_max_video($post){
    $erreur = '';
    
    $files = array();
    $debit_video_max = ciwidget_lire_meta_cle('commun','debit_video_max');
    
    if (is_array($post) AND $debit_video_max AND intval($debit_video_max)>0) {
        
        include_spip('inc/renseigner_document');
        foreach ($post as $file) {
            if (is_array($file['name'])) {
                while (count($file['name'])) {
                    $test = array(
                        'error' => array_shift($file['error']),
                        'name' => array_shift($file['name']),
                        'tmp_name' => array_shift($file['tmp_name']),
                        );
                    
                    if (!($test['error'] == 4)) {
                        if (preg_match(',\.([a-z0-9]+)(\?.*)?$,i', $test['name'], $match) and $fichier_tmp_ext = $match[1]) {
                            $fichier_tmp = $test['tmp_name'];

                            $infos = renseigner_taille_dimension_image($fichier_tmp, $fichier_tmp_ext);
                            // debit en kb/s
                            if (isset($infos['taille']) AND isset($infos['duree'])){
                                $debit = ( 8 * intval($infos['taille']) ) / ( 1000 * intval($infos['duree']) );
                                $debit = round($debit);
                                if ($debit AND $debit > intval($debit_video_max)) {
                                        $erreur = _T('ciwidget:erreur_debit_video', array('debit' => $debit,'debit_video_max' => intval($debit_video_max)));
                                        spip_unlink($fichier_tmp);
                                }
                            }
                        }
                    }
                }
            } else {
                if (!($file['error'] == 4)) {
                    if (preg_match(',\.([a-z0-9]+)(\?.*)?$,i', $file['name'], $match) and $fichier_tmp_ext = $match[1]) {
                        $fichier_tmp = $file['tmp_name'];

                        $infos = renseigner_taille_dimension_image($fichier_tmp, $fichier_tmp_ext);
                        // debit en kb/s
                        if (isset($infos['taille']) AND isset($infos['duree'])){
                            $debit = ( 8 * intval($infos['taille']) ) / ( 1000 * intval($infos['duree']) );
                            $debit = round($debit);
                            if ($debit AND $debit > intval($debit_video_max)) {
                                    $erreur = _T('ciwidget:erreur_debit_video', array('debit' => $debit,'debit_video_max' => intval($debit_video_max)));
                                    spip_unlink($fichier_tmp);
                            }
                        }
                    }
                }
            }
        }
    }
    
    return $erreur;        
}

function ciwidget_verifier_distant_debit_max_video($path='',$extension=''){
    $erreur = '';

    $debit_video_max = ciwidget_lire_meta_cle('commun','debit_video_max');

    if ($path AND $extension){
        if ($debit_video_max AND intval($debit_video_max)>0) {

            $infos = renseigner_taille_dimension_image($path, $extension, true);

            // debit en kb/s
            if (isset($infos['taille']) AND isset($infos['duree'])){
                $debit = ( 8 * intval($infos['taille']) ) / ( 1000 * intval($infos['duree']) );
                $debit = round($debit);
                if ($debit AND $debit > intval($debit_video_max)) {
                        $erreur = _T('ciwidget:erreur_debit_video', array('debit' => $debit,'debit_video_max' => intval($debit_video_max)));
                }
            }
        }
    }
    
    return $erreur;
}

?>